﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.AI;

public class DetectiveScript : MonoBehaviour
{
    //Booleans to specify if the abilities are considered 'active' or not. 
    //Active is defined as 'Ready to fire'. For example, if Penelope's search ability is 'Active', then right clicking a room will call a much quicker search.
    protected bool ab1Active = false;
    protected bool ab2Active = false;


    //What are the stats of this detective?
    public Detective DetectiveStats;

    //Reference to a UI object that defines the state the detective is in
    public Text StateText;

    //What state is the detective in? 
    public int State;

    //Get a reference to the EvidenceHandler in the level.
    EvidenceHandler _handler;

    //
    NavMeshAgent _agent;

    //Get a reference to the Notebook.
    public Notebook _notebook;

    // Use this for initialization
    virtual protected void Start()
    {
        //Init the state.
        State = 0;

        //Set the handler reference to the handler in the game world.
        _handler = FindObjectOfType<EvidenceHandler>() as EvidenceHandler;

        _agent = GetComponent<NavMeshAgent>();

        //_notebook = FindObjectOfType<Notebook>() as Notebook;
    }

    // Update is called once per frame
    void Update()
    {
        displayState();
        checkMovement();
    }

    /// <summary>
    /// Displays the state that the detective is in.
    /// </summary>
    void displayState()
    {
        switch (State)
        {
            case 0:
                StateText.text = "Idle State";
                break;
            case 1:
                break;
            case 2:
                StateText.text = "Interrogating";
                break;
            default:
                State = 0;
                break;
        }
    }

    /// <summary>
    /// Searches any room given with the detective who calls the function.
    /// </summary>
    /// <param name="r">A Reference to the room that we are attempting to search</param>
    virtual public void SearchRoom(RoomScript r)
    {
        //Sets up some local variables to calculate the time that the search is going to take.
        float timeToSearch, roomSize;

        //Calculates the size of the room given by public variables.
        roomSize = r.thisRoom.Dimensions.x * r.thisRoom.Dimensions.y;

        //Does the math to calculate search time.
        timeToSearch = ((110 - DetectiveStats.SearchAbility) / 10.0f) * roomSize;

        MoveToRoom(r);

        //Begins the coroutine that handles timing the search
        StartCoroutine(StartCountdown(timeToSearch, r));
    }

    /// <summary>
    /// Searches any room given with the detective who calls the function.
    /// </summary>
    /// <param name="r">A Reference to the room that we are attempting to search</param>
    virtual public void Interrogate(WitnessScript w)
    {

        //Test to see if the witness has already been interrogated.
        if(w.interrogated)
        {
            return;
        }

        //Calculate the time to interrogate & similarity between the detective and the witness.
        float timeToInterrogate = 180.0f;
        float similarityScale1 = (DetectiveStats.Det.Stats.WeakStrongScale - w.personality.WeakStrongScale) / 2.0f;
        float similarityScale2 = (DetectiveStats.Det.Stats.DepravedHonorableScale - w.personality.DepravedHonorableScale) / 2.0f;
        float similarityScale3 = (DetectiveStats.Det.Stats.LazyAmbitiousScale - w.personality.LazyAmbitiousScale) / 2.0f;

        float sim = (similarityScale1 + similarityScale2 + similarityScale3) / 3.0f;

        timeToInterrogate *= ((100.0f - sim) / 100.0f);

        //Begin the interrogation coroutine.
        StartCoroutine(StartCountdown(timeToInterrogate, w, sim));
        
    }

    /// <summary>
    /// A Co-Routine that handles timing the search.
    /// </summary>
    /// <param name="t">The time that the search is going to take.</param>
    /// <param name="r">Reference to the room we are searching.</param>
    /// <returns></returns>
    public IEnumerator StartCountdown(float t, RoomScript r)
    {
        float timer = t;

        //Sets state to the searching mode.
        State = 1;

        //While the timer is above zero, display the time remaining on the screen (Truncated to 2DP) and then wait that delay in this co-routine while reducing that frame-time from the timer.
        while (timer > 0)
        {
            StateText.text = "Searching... " + timer.ToString("F2") + " Seconds Remaining";
            yield return new WaitForSeconds(Time.deltaTime);
            timer -= Time.deltaTime;
        }

        Evidence[] evidenceList = r.gameObject.GetComponents<Evidence>();

        for (int i = 0; i < evidenceList.Length; i++)
        {
            if (evidenceList[i].type == Evidence.evidenceType.Clue)
            {
                _handler.foundClues.Add(evidenceList[i]);
                _notebook.AddClues(evidenceList[i]);
            }
        }
        r.searched = true;
        State = 0;
    }

    /// <summary>
    /// A Co-Routine that handles timing the interrogation.
    /// </summary>
    /// <param name="t">The time that the search is going to take.</param>
    /// <param name="w">Reference to the witness we are interrogating.</param>
    /// <param name="sim">Reference to how similar the witness is to the detective.</param>
    /// <returns></returns>
    public IEnumerator StartCountdown(float t, WitnessScript w, float sim)
    {
        float timer = t;

        w.beingQuestioned = true;
        w.makeStatement(sim);

        //Sets state to the searching mode.
        State = 2;

        //While the timer is above zero, display the time remaining on the screen (Truncated to 2DP) and then wait that delay in this co-routine while reducing that frame-time from the timer.
        while (timer > 0)
        {
            StateText.text = "Interrogating... " + timer.ToString("F2") + " Seconds Remaining";
            yield return new WaitForSeconds(Time.deltaTime);
            timer -= Time.deltaTime;
        }

        StateText.text = "Interrogation Complete!";


        Object[] evidenceList = w.gameObject.GetComponents<Evidence>();

        for (int i = 0; i < evidenceList.Length; i++)
        {
            Evidence e = evidenceList[i] as Evidence;

            if (e.type == Evidence.evidenceType.Statement)
            {
                _handler.foundStatements.Add(e);
                _notebook.AddStatements(e);
            }
        }

        w.beingQuestioned = false;

        //Resets the state.
        State = 0;
    }

    public void MoveToRoom(RoomScript room)
    {
        _agent.SetDestination(room.searchLocation.position);
    }

    public void MoveToLocation(Vector3 endPosition)
    {
        _agent.SetDestination(endPosition);
    }

    public void ToggleAb1()
    {
        ab1Active = !ab1Active;

        if (ab1Active && ab2Active)
            ab2Active = false;
    }

    public void ToggleAb2()
    {
        ab2Active = !ab2Active;

        if (ab1Active && ab2Active)
            ab1Active = false;
    }

    /// <summary>
    /// Checks whether the detective has moved
    /// </summary>
    /// <returns><c>true</c> the detective has moved from the previous frame <c>false</c> the detective hasn't moved </returns>
    public bool checkMovement()
    {
        if(gameObject.tag == "Character")
        {
            //this code can be found here: https://answers.unity.com/questions/324589/how-can-i-tell-when-a-navmesh-has-reached-its-dest.html
            if (!_agent.pathPending)
            {
                if (_agent.remainingDistance <= _agent.stoppingDistance)
                {
                    if (!_agent.hasPath || _agent.velocity.sqrMagnitude == 0f)
                    {
                        return false;
                    }
                }
            }
        }

        return true;
    }
}
